/* ------------------------------------------------------------------------------
  File: chr6dm_HMC.c
  Author: CH Robotics
  Version: 1.0
  
  Description: Functions for interacting with HMC5843 magnetic sensor
------------------------------------------------------------------------------ */ 

#include "stm32f10x.h"
#include "CHR_L3GD20.h"
#include "CHR_i2c.h"

/*******************************************************************************
* Function Name  : initializeL3G
* Input          : None
* Output         : uint8_t* status_flag
* Return         : 1 if success, 0 if fail
* Description    : Initializes the L3GD20 rate gyro
						 
*******************************************************************************/
int32_t initializeL3G( uint8_t* status_flag )
{
    uint8_t txBuf[6];
    uint8_t returnval;

	// Write i2c data
    txBuf[0] = L3G_REG_CNTRL1 | 0x80; // The "|0x80" addition causes the internal address pointer to be incremented to allow a batch write

	txBuf[1] = 0xEF;			// Highest sample rate, 30 Hz LPF cutoff frequency, normal power mode
	txBuf[2] = 0x29;			// High-pass filter config - lowest corner and normal mode
	txBuf[3] = 0x08;			// Enable data ready interrupt and set to push-pull mode
	txBuf[4] = 0x30 | 0x80;		// 2000 deg/s full-scale range, little-endian data, and block data update until all data read (0x80)
	txBuf[5] = 0x02;			// Disable FIFO and HPF

    returnval = i2cBatchWrite( L3G_SLAVE_ADDRESS7, txBuf, 6 );

	// Now read it back to see if it worked...
	i2cBatchRead( L3G_SLAVE_ADDRESS7, L3G_REG_CNTRL1 | 0x80, g_i2cRxBuf, 5 );

	int index;
	for( index = 0; index < 5; index++ )
	{
		if( txBuf[index+1] != g_i2cRxBuf[index] )
		{
			return 1;
		}
	}
	
    return returnval;
}

/*******************************************************************************
* Function Name  : getL3GData
* Input          : None
* Output         : uint8_t* i2cBuf
* Return         : 1 if success, 0 if fail
* Description    : 
*******************************************************************************/
int32_t getL3GData( )
{
    int32_t returnval;
    
    i2cBatchRead( L3G_SLAVE_ADDRESS7, L3G_REG_OUT_TEMP | 0x80, g_i2cRxBuf, 8 );

    return 1;
}
