/* ------------------------------------------------------------------------------
  File: CHR_spi.h
  Author: CH Robotics
  Version: 1.0
  
  Description: Code for working with SPI bus
------------------------------------------------------------------------------ */ 

#ifndef __CHR_SPI_H
#define __CHR_SPI_H

#include "stm32f10x.h"
#include "stm32f10x_spi.h"
#include "UM6_config.h"

#define SPI_SLAVE                    SPI1
#define SPI_SLAVE_CLK                RCC_APB2Periph_SPI1
#define SPI_SLAVE_GPIO_CLK           GPIO_Pin_12
#define SPI_SLAVE_IRQn               SPI1_IRQn

#define SPI_STATE_READ 0
#define SPI_STATE_WRITE 1
#define SPI_STATE_WAITING 2

#define SPI_READ_CMD 0
#define SPI_WRITE_CMD 1


// Default value for gSPI_Curr_Address when an address hasn't 
// been provided for writing/reading
// (using the same value as the unknown address register)
#define SPI_NEED_ADDRESS (CHR_UNKNOWN_ADDRESS)

// spi_fsm_state keeps track of the device performing batch read, write, etc
extern uint32_t gSPI_Buffer;

// spi_fsm_state keeps track of the device performing batch read, write, etc
extern uint8_t gSPI_State;

// the address that is being read from (even though the contents are probably
// already in the buffer by the time this could be read
extern uint8_t gSPI_Curr_Address;

// Bytes shifted in or out  during a read/write 
// Used to keep track of which part of the buffer to use
extern uint8_t gSPI_Bytes_Shifted;

void SPI_ResetFSM();

void SPI1_IRQHandler(void);

#endif
