/*
 * EventScheduler.h
 *
 * Created: 4/10/2014 12:23:23 PM
 *  Author: Caleb
 */ 


#ifndef EVENTSCHEDULER_H_
#define EVENTSCHEDULER_H_

#include <math.h>

// Number of events that can be scheduled all at once
#define MAX_EVENTS	25
#define EVENT_TIME_THRESHOLD	5e-3f
#define COUNTER_FREQUENCY		10985.6576137f
#define COUNTER_PERIOD			0.00009102777f
#define COUNTER_OVERFLOW_TIME   5.96559644443f

//Wrapping for the GPS and system timer.  GPS wraps to 1 day by default. Set to at least the value of COUNTER_OVERFLOW_TIME.
#define SECONDS_IN_EPOCH        86400.0f
#define SECONDS_IN_EPOCHd2      (SECONDS_IN_EPOCH/2.0f)
#define GPS_TIME_DELAY_INTERVAL 1.2f

// Structure containing user args for inclusion in the callback function
typedef  struct __event_user_args
{
	uint32_t val1;
	uint32_t val2;
	uint32_t val3;
	uint32_t val4;
} UserArgs;

// Structure defining the properties of an event
typedef struct __event_properties
{
	uint32_t event_id;
	uint8_t is_periodic;
	uint8_t is_COM_event;
	float period;
	void (*callback_func)(float system_time, UserArgs user_args);
	UserArgs user_args;
} EventProperties;

// Structure encapsulating all data needed to control event handling
typedef struct __event
{
	EventProperties properties;
	float last_event_time;
} Event;

// Structure holding values used to set UTC time based on GPS UTC time and PPS signal.
// Solve Ax = y to find c0, c1,
// then t_utc_k = c0 + c1*t_sys_k
//
// Values are doubles to keep microsecond resolution in the calculations
typedef struct _UTCTiming
{
    // y = [ t_utc_k; t_utc_k_d1 ]
    double t_utc_k;     // Time from GPS at PPS
    double t_utc_k_d1;
    
    // A = [ 1, t_sys_k;
    //       1, t_sys_k_d1 ];
    double t_sys_k;
    double t_sys_k_d1;

    // x = [ c0, c1 ];
    float c0;
    float c1;

    float time_of_PPS; // Arrival time of PPS.
} UTC_Timing;

extern volatile UTC_Timing g_UTC_timing;

extern volatile float g_sys_x0;

// Event scheduler function calls
void init_event_scheduler( void );
void start_event_scheduler( void );
void stop_event_scheduler( void );
uint32_t add_event( EventProperties* event );
uint32_t remove_event( uint32_t event_index );
uint32_t remove_event_by_id( uint32_t id );
void clear_all_events( void );
void clear_all_COM_events( void );

float get_system_time_uncal( void );
float get_UTC_time( void );

void init_gps_time_struct( void );
void compute_time_calibration( void );

float unwrap_delta_T(float delta_T);

#endif /* EVENTSCHEDULER_H_ */