/* ------------------------------------------------------------------------------
  File: chr6dm_startup.c
  Author: CH Robotics
  Version: 1.0
  
  Description: Function definitions for AHRS initialization
------------------------------------------------------------------------------ */ 
#include "stm32f10x.h"
#include "UM6_startup.h"
#include "UM6_usart.h"
#include "CHR_i2c.h"
#include "CHR_spi.h"

uint8_t gClockInUse;

/*******************************************************************************
* Function Name  : Initialize_IMU
* Input          : None
* Output         : None
* Return         : None
* Description    : Initializes the system clock and all peripherals of the CHR-6d
*******************************************************************************/
void Initialize_UM6( void )
{
    ADC_InitTypeDef ADC_InitStructure;
    DMA_InitTypeDef DMA_InitStructure;
    TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;
    I2C_InitTypeDef   I2C_InitStructure;
    EXTI_InitTypeDef EXTI_InitStructure;
     
    // System Clocks Configuration
    RCC_Configuration();

    // NVIC configuration
    NVIC_Configuration();

    // Configure the GPIO ports 
    GPIO_Configuration();

    // Setup serial port with 115200 baud	 
    USART1_Configuration(115200);
    USART2_Configuration(115200);

    // Set up timer2 to function as the system clock.  Nominal clock rate is 72e6/(prescaler + 1)
    // A prescaler of 0x1999 thus yields a nominal clock rate of 10985.6576137 Hz.  With an overflow period of
    // 0xFFFF, we should overflow once every 5.96559644443 seconds.
    TIM_TimeBaseStructure.TIM_Period = 0xFFFF;
    TIM_TimeBaseStructure.TIM_Prescaler = 0x1999;
    TIM_TimeBaseStructure.TIM_ClockDivision = 0;
    TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;

    TIM_TimeBaseInit(TIM2, &TIM_TimeBaseStructure);
    TIM_ClearITPendingBit(TIM2, TIM_IT_Update);
    TIM_ITConfig(TIM2, TIM_IT_Update, ENABLE);

    // Configure Timer4 to control gyro/accel sample rate
    // With a prescaler of 72, the timer nominally increments once per microsecond.
    TIM_TimeBaseStructure.TIM_Period = 2000;
    TIM_TimeBaseStructure.TIM_Prescaler = 72;
    TIM_TimeBaseStructure.TIM_ClockDivision = 0;
    TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;

    TIM_TimeBaseInit(TIM4, &TIM_TimeBaseStructure);
    TIM_ITConfig(TIM4, TIM_IT_Update, ENABLE);

    // SPI_SLAVE1 configuration 
    SPI_InitTypeDef SPI_InitStructure;
    SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex; //Full Duplex
    SPI_InitStructure.SPI_Mode = SPI_Mode_Slave; //Slave
    SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b; //8-bit interface
    SPI_InitStructure.SPI_CPOL = SPI_CPOL_Low;//Active Low (clock polarity)
    SPI_InitStructure.SPI_CPHA = SPI_CPHA_1Edge;//Clock on First Edge (clock phase)
    SPI_InitStructure.SPI_NSS = SPI_NSS_Hard; //Hardware slave select
    SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_2;//Not a master, but need to initialize this
    SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;//Most Significant Bit sent first
    SPI_Init(SPI_SLAVE, &SPI_InitStructure);   

    // Enable SPI_SLAVE RXNE interrupt 
    SPI_I2S_ITConfig(SPI_SLAVE, SPI_I2S_IT_RXNE, ENABLE);

    // Enable SPI Slave
    SPI_Cmd(SPI_SLAVE, ENABLE);

    // Set the spi finite state machine to waiting and clear address
    SPI_ResetFSM();
     
    // Enable external interrupt 12 (DRDY pin for rate gyros)
    // Map PA12 to EXTI 12
    GPIO_EXTILineConfig(GPIO_PortSourceGPIOA, GPIO_PinSource12);

}

/*******************************************************************************
* Function Name  : USART_Init( int baud )
* Description    : Configures the USART at the selected baud rate
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void USART1_Configuration(int baud)
{
	 USART_InitTypeDef USART_InitStructure;
	 DMA_InitTypeDef DMA_InitStructure;
	 
	 // USART1 RX DMA1 Channel (triggered by USART1 Rx event) Config 
	 DMA_DeInit(DMA1_Channel5);
	 DMA_StructInit(&DMA_InitStructure);
	 DMA_InitStructure.DMA_PeripheralBaseAddr = USART1_DR_Base;
	 DMA_InitStructure.DMA_MemoryBaseAddr = (uint32_t)gRXBuf;
	 DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralSRC;
	 DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
	 DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
	 DMA_InitStructure.DMA_BufferSize = RX_BUF_SIZE;
	 DMA_InitStructure.DMA_Mode = DMA_Mode_Circular;
	 DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;
	 DMA_InitStructure.DMA_Priority = DMA_Priority_VeryHigh;
	 
	 DMA_Init( DMA1_Channel5, &DMA_InitStructure );
	 
	 // USART1 TX DMA1 channel config (triggered by USART1 Tx event)
	 DMA_DeInit(DMA1_Channel4);
	 DMA_InitStructure.DMA_PeripheralBaseAddr = USART1_DR_Base;
	 DMA_InitStructure.DMA_MemoryBaseAddr = (uint32_t)gTXBuf;
	 DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralDST;
	 DMA_InitStructure.DMA_BufferSize = TX_BUF_SIZE;		// This will need to be set again
	 DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
	 DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
	 DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;
	 DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;
	 DMA_InitStructure.DMA_Mode = DMA_Mode_Normal;
	 DMA_InitStructure.DMA_Priority = DMA_Priority_VeryHigh;
	 DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;
	 DMA_Init(DMA1_Channel4, &DMA_InitStructure);
	 
	 // Enable TX DMA transfer complete interrupt
	 DMA_ITConfig( DMA1_Channel4, DMA_IT_TC, ENABLE );
	 
	 // Disable USART1
    USART_Cmd(USART1, DISABLE);
	 
	 // USART1  ------------------------------------------------------
    /* USART1 is configured as followS:
        - BaudRate = 'baud'  
        - Word Length = 8 Bits
        - One Stop Bit
        - No parity
        - Hardware flow control disabled (RTS and CTS signals)
        - Receive and transmit enabled
    */
    USART_InitStructure.USART_BaudRate = baud;
    USART_InitStructure.USART_WordLength = USART_WordLength_8b;
    USART_InitStructure.USART_StopBits = USART_StopBits_1;
    USART_InitStructure.USART_Parity = USART_Parity_No;
    USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
    USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;

    // Configure USART
    USART_DeInit(USART1);
	 USART_Init(USART1, &USART_InitStructure);
	 
    // Enable USART1 Receive and Transmit interrupts
//    USART_ITConfig(USART1, USART_IT_RXNE, ENABLE);
//    USART_ITConfig(USART1, USART_IT_TC, ENABLE);

	 // Enable USART1 TX and RX DMA requests
	 USART_DMACmd(USART1, USART_DMAReq_Rx, ENABLE);
	 USART_DMACmd(USART1, USART_DMAReq_Tx, ENABLE);
	 
	 // Enable DMA1 USART RX channel
	 DMA_Cmd(DMA1_Channel5, ENABLE);
	 
	 // Enable DMA1 USART TX channel (this should be enabled when there is data to transmit over the
	 // USART.  Transmission will start automatically.
//	 DMA_Cmd(DMA1_Channel4, ENABLE);

	 // Set global flags that indicate the USART status
	 gTXBusy = 0;
	 gRXBufPtr = 0;
	 gTXBufPtr = 0;
	 gUSART_State = USART_STATE_WAIT;
    
    // Enable USART1
    USART_Cmd(USART1, ENABLE);
}

/*******************************************************************************
* Function Name  : USART_Init( int baud )
* Description    : Configures the USART at the selected baud rate
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void USART2_Configuration(int baud)
{
	 USART_InitTypeDef USART_InitStructure;
	 DMA_InitTypeDef DMA_InitStructure;
	 
	 // USART2 RX DMA1 Channel (triggered by USART2 Rx event) Config 
	 DMA_DeInit(DMA1_Channel6);
	 DMA_StructInit(&DMA_InitStructure);
	 DMA_InitStructure.DMA_PeripheralBaseAddr = USART2_DR_Base;
	 DMA_InitStructure.DMA_MemoryBaseAddr = (uint32_t)gGPSBuf;
	 DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralSRC;
	 DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
	 DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
	 DMA_InitStructure.DMA_BufferSize = RX_BUF_SIZE;
	 DMA_InitStructure.DMA_Mode = DMA_Mode_Circular;
	 DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;
	 DMA_InitStructure.DMA_Priority = DMA_Priority_VeryHigh;
	 
	 DMA_Init( DMA1_Channel6, &DMA_InitStructure );
	 
	 // Disable USART2
     USART_Cmd(USART2, DISABLE);
	 
	 // USART1  ------------------------------------------------------
    /* USART1 is configured as follows:
        - BaudRate = 'baud'  
        - Word Length = 8 Bits
        - One Stop Bit
        - No parity
        - Hardware flow control disabled (RTS and CTS signals)
        - Receive enabled, but not transmit
    */
    USART_InitStructure.USART_BaudRate = baud;
    USART_InitStructure.USART_WordLength = USART_WordLength_8b;
    USART_InitStructure.USART_StopBits = USART_StopBits_1;
    USART_InitStructure.USART_Parity = USART_Parity_No;
    USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
    USART_InitStructure.USART_Mode = USART_Mode_Rx;

    // Configure USART
    USART_DeInit(USART2);
    USART_Init(USART2, &USART_InitStructure);

    // Enable USART2 RX DMA request
    USART_DMACmd(USART2, USART_DMAReq_Rx, ENABLE);

    // Enable DMA1 USART RX channel
    DMA_Cmd(DMA1_Channel6, ENABLE);


    // Set global flags that indicate the USART status
    gGPSBufPtr = 0;
    
    // Enable USART2
    USART_Cmd(USART2, ENABLE);
}

/*******************************************************************************
* Function Name  : RCC_Configuration
* Description    : Configures the different system clocks.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void RCC_Configuration(void)
{
	 ErrorStatus HSEStartUpStatus;
	 
	 RCC_DeInit();
	 
    /* Disable HSI */
//    RCC_HSICmd(ENABLE);
	 
	 /* Enable external oscillator */
	 RCC_HSEConfig(RCC_HSE_ON);
	 
	 /* Wait till HSE is ready */
	 HSEStartUpStatus = RCC_WaitForHSEStartUp();
//	 while( RCC_WaitForHSEStartUp() != SUCCESS );
	 
	 /* Enable Prefetch Buffer */
    FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);

    /* Flash 2 wait state */
    FLASH_SetLatency(FLASH_Latency_2);

    /* HCLK = SYSCLK */
    RCC_HCLKConfig(RCC_SYSCLK_Div1);

    /* PCLK2 = HCLK/1 */
    RCC_PCLK2Config(RCC_HCLK_Div1); 

    /* PCLK1 = HCLK/2 */
    RCC_PCLK1Config(RCC_HCLK_Div2);

    /* ADCCLK = PCLK2/6 */
    RCC_ADCCLKConfig(RCC_PCLK2_Div6); 

	 if( HSEStartUpStatus == SUCCESS )
	 {
		  gClockInUse = 'E';
		  RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9);	// 8 / 1 * 9 = 72 Mhz
	 }
	 else
	 {
		  gClockInUse = 'I';
		  RCC_PLLConfig(RCC_PLLSource_HSI_Div2, RCC_PLLMul_16);
	 }
	 
    /* Enable PLL */ 
    RCC_PLLCmd(ENABLE);

    /* Wait till PLL is ready */
    while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET)
     {;}

    /* Select PLL as system clock source */
    RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);

    /* Wait till PLL is used as system clock source */
    while(RCC_GetSYSCLKSource() != 0x08)
     {;}

    /* Enable peripheral clocks --------------------------------------------------*/

    /* Enable GPIO and UART clocks */
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_GPIOB | RCC_APB2Periph_AFIO | RCC_APB2Periph_USART1, ENABLE);
	RCC_APB1PeriphClockCmd( RCC_APB1Periph_USART2, ENABLE );
    
    // Enable ADC1 and ADC2 clocks
    RCC_APB2PeriphClockCmd( RCC_APB2Periph_ADC1 | RCC_APB2Periph_ADC2, ENABLE );
                             
    // Enable DMA clock
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);
			
    // Enable TIM2 clock
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE);
			
    // Enable TIM3 clock
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3, ENABLE);
           
    // Enable TIM4 clock
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM4, ENABLE);
           
    // Enable i2c1 clock
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_I2C1, ENABLE);
     
    // Enable SPI_SLAVE and GPIO clocks for SPI_SLAVE
    RCC_APB2PeriphClockCmd(SPI_SLAVE_GPIO_CLK | SPI_SLAVE_CLK, ENABLE);

}

/*******************************************************************************
* Function Name  : GPIO_Configuration
* Input          : None
* Output         : None
* Return         : None
* Description    : Configures the GPIO ports on the IMU
*******************************************************************************/
void GPIO_Configuration(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;

	// Configure USART1 Rx as input, internal pullup
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	// Configure USART1 Tx as alternate function push-pull
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_9;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	// Configure USART2 Rx as input with internal pull-up
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_3;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	/*	We'll do this later, depending on whether this pin should be an output or a PPS input
	// Configure USART2 Tx as alternate function push-pull
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_2;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
	*/

	// LED output pin
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	// Configure PA12 as input floating (accel interrupt input)
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_12;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	// Configure I2C1 pins: SCL and SDA
	GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_6 | GPIO_Pin_7;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_OD;
	GPIO_Init(GPIOB, &GPIO_InitStructure);

	// Remap SPI1 pins and turn off JTAG so that we can use SPI
	GPIO_PinRemapConfig(GPIO_Remap_SWJ_JTAGDisable, ENABLE);
	GPIO_PinRemapConfig(GPIO_Remap_SWJ_Disable , ENABLE);
	GPIO_PinRemapConfig(GPIO_Remap_SPI1, ENABLE);
	
	//PB3->SPI1 SCK    Input, floating
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_3;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
	GPIO_Init(GPIOB, &GPIO_InitStructure);

	//PB4->SPI1 MISO    Alternate Function Push Pull
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_4;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
	GPIO_Init(GPIOB, &GPIO_InitStructure);

	//PB5->SPI1 MOSI    Input, floating
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_5;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
	GPIO_Init(GPIOB, &GPIO_InitStructure);

	//PA15->SPI1 SS    Input, internal pull-up
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_15;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
}   


/*******************************************************************************
* Function Name  : NVIC_Configuration
* Input          : None
* Output         : None
* Return         : None
* Description    : Configures the vectored interrupt table on the IMU
*******************************************************************************/
void NVIC_Configuration(void)
{
	NVIC_InitTypeDef NVIC_InitStructure;

	/* Configure the NVIC Preemption Priority Bits */  
	NVIC_PriorityGroupConfig(NVIC_PriorityGroup_0);

	/*	DISABLED - DMA IS USED INSTEAD
	// Enable the USART1 Interrupt (only peripheral with higher priority is the i2c bus)
	NVIC_InitStructure.NVIC_IRQChannel = USART1_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);
	*/

	// Enable the DMA1 channel4 interrupt
	NVIC_InitStructure.NVIC_IRQChannel = DMA1_Channel4_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 2;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);

	/*
	// Enable the DMA1 channel6 interrupt
	NVIC_InitStructure.NVIC_IRQChannel = DMA1_Channel6_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);

	// Enable the DMA1 channel7 interrupt
	NVIC_InitStructure.NVIC_IRQChannel = DMA1_Channel7_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);
	*/

	// Enable the TIM2 global Interrupt and set at lowest priority.
	// This is used to tell the MCU to transmit newest state data over the UART
	NVIC_InitStructure.NVIC_IRQChannel = TIM2_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0x0F;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0x0F;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);

	// Enable the TIM4 global Interrupt
	// This is used to control the sample rate of the gyros
	NVIC_InitStructure.NVIC_IRQChannel = TIM4_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0x00;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0x01;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);

	/* DISABLED TO FOR DMA USAGE
	// Configure and enable I2C1 event interrupt (don't let inaything interfere with i2c communication)
	NVIC_InitStructure.NVIC_IRQChannel = I2C1_EV_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);
	*/

	// Enable and set EXTI Interrupt for handling PPS input
	/* We'll do this later, when the pin itself is configured
	NVIC_InitStructure.NVIC_IRQChannel = EXTI2_IRQn;// EXTI15_10_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;

	NVIC_Init(&NVIC_InitStructure);

	*/

	// Enable the SPI1 Slave Interrupt
	NVIC_InitStructure.NVIC_IRQChannel = SPI_SLAVE_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);
}
